/***********************************************************************************
 *		  																								     *
 *	File:			Generic.cpp.			  				 											  *
 *																											  *
 * Contents: 	Interface and implementation of generic functions.						  *
 *																											  *
 * Purpose:  	-																						  *
 *																											  *
 * Remarks:    -												 										  *
 *																											  *
 * Originator: Michael Mogensen, Hilleroed, Denmark.										  *
 *																											  *
 *	E-mail:		mogensen@post6.tele.dk.	      												  *
 *																											  *
 * Compiler:	Borland C++ 5.01.																	  *
 *																											  *
 * Period:		00.00.00 - 00.00.00 																  *
 *																											  *
 ***********************************************************************************/

#ifndef _Generic_h
#define _Generic_h

template <class T>
T Abs(const T &X)
// Return absoulte value of X.
{ T Y = X < (T)0 ? -X : X; return Y; }


template <class T>
T GetRandomObjectBetween(const T Min, const T Max)
// Return random object in the set [Min, Max] where Min <= Max.
{
   T Random = Min + (T)random((int)Max - (int)Min + 1);
	return Random;
}

template <class T>
void SetCyclic(T &A, const T B, const T C, const T D)
// Cyclic increse or decrese A with the value of B to fit the interval [C, D].
{
	A += B;
	if(A < C)
   	A = D;
   else
   	if(A > D)
      	A = C;
}

template <class T>
void SetLinear(T &A, const T B, const T C)
// Linear increse or decrese A with the value of B until C are reached.
{
   A += B;
   const int iB = (int)B;
	if((iB > 0 && A > C) || (iB < 0 && A < C))
     	A = C;
}

template <class T>
void SwapObjects(T &A, T &B)
// Swap two values of type T.
{
	T A_= A;
	A = B;
   B = A_;
}

template <class T_A, class T_B>
void SupressObjectInDescreteRange(T_A &X, const T_B Bottom, const T_B Top, const T_B Step)
// Force object to be in the set [Bottom, Top], but in a descrete way.
{
   if(X > Top)
   	X = Top;
   else
   	if(X < Bottom)
      	X = Bottom;
   	else
      	X = Bottom + Step * ((X - Bottom) / Step);
}

template <class T_A, class T_B>
void SupressObjectInRange(T_A &X, const T_B Bottom, const T_B Top)
// Force object to be in the set [Bottom, Top].
{
   if(X > Top)
   	X = Top;
   else
   	if(X < Bottom)
      	X = Bottom;
}

template <class T_A, class T_B>
const T_A SupressedObjectInRange(const T_A &X, const T_B Bottom, const T_B Top)
// Return object in the set [Bottom, Top].
{
   if(X > Top)
   	return Top;
   else
   	if(X < Bottom)
      	return Bottom;
   	else
      	return X;
}

#endif

