//----------------------------------------------------------------------------
//  Project Lstwndex
//  Tenermerx Software
//  Copyright  1997 Tenermerx Software.
//
//  SUBSYSTEM:    Lstwndex Application
//  FILE:         lwwindow.cpp
//  AUTHOR:       Christopher Kohlhoff
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TLWWindow (TWindow).
//  This source file has been updated to work for Borland C++ 5.02.
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/listwind.h>
#include <owl/imagelst.h>
#include <classlib/time.h>
#include <stdio.h>
#include "lwapp.h"
#include "lwwindow.h"



const int kListId = 1;


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TLWWindow, TWindow)
//{{TLWWindowRSP_TBL_BEGIN}}
  EV_COMMAND_AND_ID(CM_VIEWLARGEICONS, CmViewChange),
  EV_COMMAND_AND_ID(CM_VIEWSMALLICONS, CmViewChange),
  EV_COMMAND_AND_ID(CM_VIEWLIST, CmViewChange),
  EV_COMMAND_AND_ID(CM_VIEWDETAILS, CmViewChange),
  EV_COMMAND_ENABLE(CM_VIEWLARGEICONS, CeViewChange),
  EV_COMMAND_ENABLE(CM_VIEWSMALLICONS, CeViewChange),
  EV_COMMAND_ENABLE(CM_VIEWLIST, CeViewChange),
  EV_COMMAND_ENABLE(CM_VIEWDETAILS, CeViewChange),
  EV_LVN_COLUMNCLICK(kListId, ListColumnClick),
  EV_NM_DBLCLK(kListId, ListDoubleClick),
  EV_WM_SIZE,
//{{TLWWindowRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TLWWindow Implementation}}


//--------------------------------------------------------
// TLWWindow
// ~~~~~~~~~~
// Construction/Destruction handling.
//
TLWWindow::TLWWindow(TWindow* parent, const char far* title, TModule* module)
:
  TWindow(parent, title, module),
  ViewBy(LVS_REPORT),
  SortUp(true),
  Column(-1)
{
  // create the list window (the size is unimportant - see EvSize)
  //
  List = new TListWindow(this, kListId, 0, 0, 0, 0, module);
  List->Attr.Style |= ViewBy;

  // create the imagelists for the list window from a bitmap
  //
  SmallImg = new TImageList(*GetApplication(), TResId(IDB_SMALLIMAGES), 16, 1,
                            TColor::LtGreen, IMAGE_BITMAP, 0);
  LargeImg = new TImageList(*GetApplication(), TResId(IDB_LARGEIMAGES), 32, 1,
                            TColor::LtGreen, IMAGE_BITMAP, 0);
}


TLWWindow::~TLWWindow()
{
  Destroy();
  delete SmallImg;
  delete LargeImg;
}


void TLWWindow::EvSize(uint sizeType, TSize& size)
{
  TWindow::EvSize(sizeType, size);

  // resize the list to fill the client area
  //
  List->MoveWindow(GetClientRect(), true);
}


//
// Add the columns to the list box
//
void TLWWindow::AddListColumns()
{
  List->InsertColumn(0, TListWindColumn("Name", 200));
  List->InsertColumn(1, TListWindColumn("Size", 80, TListWindColumn::Right));
  List->InsertColumn(2, TListWindColumn("Type", 120));
  List->InsertColumn(3, TListWindColumn("Modified", 200));
}


//
// Add the columns to the list box
//
void TLWWindow::AddListItems()
{
  // This code is for Borland C++ 5.02. If you are using
  // Borland C++ 5.01, see below.
  //

  HANDLE findHandle;
  WIN32_FIND_DATA findData;
  char text[256];
  SYSTEMTIME st;

  // remove all of the items from the list
  //
  List->DeleteAllItems();

  // open the current directory for reading
  //
  if ((findHandle = FindFirstFile(".\\*.*", &findData)) == INVALID_HANDLE_VALUE)
    return;

  // add each of the files in the directory
  //
  do
  {
    // add the name
    //
    TListWindItem item(findData.cFileName);
    item.SetImageIndex(findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? 1 : 0);
    int index = List->AddItem(item);

    // add the size of the item
    //
    itoa((findData.nFileSizeHigh * MAXDWORD) + findData.nFileSizeLow, text, 10);
    List->SetItemText(index, 1, text);

    // add the type of the item
    //
    if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      List->SetItemText(index, 2, "Directory");
    else
      List->SetItemText(index, 2, "File");

    // add the time modified
    //
    FileTimeToSystemTime(&findData.ftLastWriteTime, &st);
    TTime time(TDate(st.wDay, st.wMonth, st.wYear), st.wHour, st.wMinute, st.wSecond);
    List->SetItemText(index, 3, time.AsString().c_str());
  }
  while (FindNextFile(findHandle, &findData));

  FindClose(findHandle);

#if 0
  // This is the code for Borland C++ 5.01
  //

  HANDLE findHandle;
  WIN32_FIND_DATA findData;
  char text[256];
  SYSTEMTIME st;

  // remove all of the items from the list
  //
  List->DeleteAllItems();

  // open the current directory for reading
  //
  if ((findHandle = FindFirstFile(".\\*.*", &findData)) == INVALID_HANDLE_VALUE)
    return;

  // add each of the files in the directory
  //
  do  {
    // add the name
    //
    TListWindItem item(findData.cFileName);
    item.SetImageIndex(findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? 1 : 0);
    List->AddItem(item);

    // add the size of the item
    //
    itoa((findData.nFileSizeHigh * MAXDWORD) + findData.nFileSizeLow, text, 10);
    item.SetText(text);
    List->SetItem(item, -1, 1);

    // add the type of the item
    //
    if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      item.SetText("Directory");
    else
      item.SetText("File");
    List->SetItem(item, -1, 2);

    // add the time modified
    //
    FileTimeToSystemTime(&findData.ftLastWriteTime, &st);
    TTime time(TDate(st.wDay, st.wMonth, st.wYear), st.wHour, st.wMinute, st.wSecond);
    strcpy(text, time.AsString().c_str());
    item.SetText(text);
    List->SetItem(item, -1, 3);
  } while (FindNextFile(findHandle, &findData));

  FindClose(findHandle);
#endif
}


void TLWWindow::SetupWindow()
{
  TWindow::SetupWindow();

  // set the image lists for use with the list window
  //
  List->SetImageList(*SmallImg, TListWindow::Small);
  List->SetImageList(*LargeImg, TListWindow::Normal);

  // add some interesting stuff to the list
  //
  AddListColumns();
  AddListItems();
}


//
// change how the items in the list are viewed
//
void TLWWindow::CmViewChange(WPARAM id)
{
  int newViewBy;
  switch (id)
  {
    case CM_VIEWLARGEICONS: newViewBy = LVS_ICON; break;
    case CM_VIEWSMALLICONS: newViewBy = LVS_SMALLICON; break;
    case CM_VIEWLIST:       newViewBy = LVS_LIST; break;
    case CM_VIEWDETAILS:    newViewBy = LVS_REPORT; break;
  }
  List->ModifyStyle(ViewBy, newViewBy);
  ViewBy = newViewBy;
}


//
// determine if the menu item should be checked to reflect
// how the items in the list are being viewed
//
void TLWWindow::CeViewChange(TCommandEnabler &tce)
{
  bool check;
  switch (tce.GetId())
  {
    case CM_VIEWLARGEICONS: check = (ViewBy == LVS_ICON); break;
    case CM_VIEWSMALLICONS: check = (ViewBy == LVS_SMALLICON); break;
    case CM_VIEWLIST:       check = (ViewBy == LVS_LIST); break;
    case CM_VIEWDETAILS:    check = (ViewBy == LVS_REPORT); break;
  }
  tce.SetCheck(check);
}


//
// Our list window sorting base class
//
class TMyComparator : public TLwComparator
{
  public:
    TMyComparator(TListWindow* list, int col, bool up)
      : List(list), Col(col), Up(up) {}
  protected:
    TListWindow* List;
    int Col;
    bool Up;
};


//
// Our comparator class for sorting strings
//
class TMyStringComparator : public TMyComparator
{
  public:
    TMyStringComparator(TListWindow* l, int c, bool u) : TMyComparator(l, c, u) {}
    virtual int Compare(uint32 item1, uint32 item2, uint32 lParam) const;
};

int TMyStringComparator::Compare(uint32 item1, uint32 item2, uint32) const
{
  TListWindItem lwItem1(*List, item1, Col);
  TListWindItem lwItem2(*List, item2, Col);
  int val = strcmp(lwItem1.GetText(), lwItem2.GetText());
  return (Up ? val : -val);
}


//
// Our comparator class for sorting integers
//
class TMyIntComparator : public TMyComparator
{
  public:
    TMyIntComparator(TListWindow* l, int c, bool u) : TMyComparator(l, c, u) {}
    virtual int Compare(uint32 item1, uint32 item2, uint32 lParam) const;
};

int TMyIntComparator::Compare(uint32 item1, uint32 item2, uint32) const
{
  TListWindItem lwItem1(*List, item1, Col);
  TListWindItem lwItem2(*List, item2, Col);
  int val = atoi(lwItem1.GetText()) - atoi(lwItem2.GetText());
  return (Up ? val : -val);
}


//
// sort the items in the listbox
//
void TLWWindow::ListColumnClick(TLwNotify& notify)
{
  // check if this is the same column as we sorted last
  // time, and if so change the direction of the sort
  //
  SortUp = (notify.iSubItem == Column ? !SortUp : true);
  Column = notify.iSubItem;

  // set all itemdatas for the first column to be their index
  //
  for (int i = 0, imax = List->GetItemCount(); i < imax; i++)
  {
    TListWindItem item(*List, i, 0);
    item.SetItemData(i);
    List->SetItem(item);
  }
    
  // sort the items in the list (use int comparator for the
  // size column, string comparator for all others
  //
  if (Column == 1)
    List->SortItems(TMyIntComparator(List, Column, SortUp));
  else
    List->SortItems(TMyStringComparator(List, Column, SortUp));
}


void TLWWindow::ListDoubleClick()
{
  // If you are using Borland C++ v5.02 find the
  // selected item using this method.
  //
  int selItem;
  if (List->GetSelIndexes(&selItem, 1) == 1)
  {
    TListWindItem nameItem(*List, selItem, 0);
    TListWindItem typeItem(*List, selItem, 2);

    // if the item is a folder, change to it and refresh
    //
    if (!strcmp(typeItem.GetText(), "Directory"))
    {
      SetCurrentDirectory(nameItem.GetText());
      AddListItems();
    }
  }

  // Otherwise, if you are using v5.01, use this
  // method to find the first item that is selected
  //
#if 0
  for (int i = 0, imax = List->GetItemCount(); i < imax; i++)
    if (List->GetItemState(i, TListWindItem::Selected))
    {
      TListWindItem nameItem(*List, i, 0);
      TListWindItem typeItem(*List, i, 2);

      // if the item is a folder, change to it and refresh
      //
      if (!strcmp(typeItem.GetText(), "Directory"))
      {
        SetCurrentDirectory(nameItem.GetText());
        AddListItems();
      }

      break;
    }
#endif
}

