//----------------------------------------------------------------------------
//  Project BmpMenuSDIApp
//  
//  Copyright  1998. All Rights Reserved.
//
//  SUBSYSTEM:    BmpMenuSDIApp Application
//  FILE:         bmpmenusdiapp.h
//  AUTHOR:       Dieter Windau
//
//  OVERVIEW
//  ~~~~~~~~
//  Class definition for TBmpMenuSDIApp (TApplication).
//
//----------------------------------------------------------------------------
#if !defined(bmpmenusdiapp_h)              // Sentry, use file only if it's not already included.
#define bmpmenusdiapp_h

#include <owl/controlb.h>
#include <owl/docking.h>
#include <owl/mailer.h>
#include <owl/printer.h>
#include <owl/rcntfile.h>


#include "sdi/apxprint.h"
#include "sdi/apxprev.h"

#include "sdi/bmpmenusdiapp.rh"            // Definition of all resources.

class TBmpMenu;

//
// FrameWindow must be derived to override Paint for Preview and Print.
//
//{{TDecoratedFrame = TSDIDecFrame}}
class TSDIDecFrame : public TDecoratedFrame {
  public:
    TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection = false, TModule* module = 0);
    ~TSDIDecFrame();

    TBmpMenu* GetBmpMenu() { return BmpMenu; }

    virtual int GetHintText(uint id, char* buf, int size, THintText hintType) const;
    
  protected:
    virtual void SetupWindow();
    
    void  EvInitMenu(HMENU hMenu);
    void  EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);
    void  EvMeasureItem(uint ctrlId, MEASUREITEMSTRUCT far& measureInfo);

    int32 EvMenuChar(uint nChar, uint menuType, HMENU hMenu);

  protected:
	  TBmpMenu* BmpMenu;

  DECLARE_RESPONSE_TABLE(TSDIDecFrame);    
};    //{{TSDIDecFrame}}


//{{TApplication = TBmpMenuSDIApp}}
class TBmpMenuSDIApp : public TApplication, public TRecentFiles {
  private:

  private:
    void SetupSpeedBar(TDecoratedFrame* frame);

  public:
    TBmpMenuSDIApp();
    virtual ~TBmpMenuSDIApp();

    void CreateGadgets(TDockableControlBar* cb, bool server = false);
    THarbor*        ApxHarbor;

    // Public data members used by the print menu commands and Paint routine in MDIChild.
    //
    TPrinter*       Printer;                            // Printer support.
    int             Printing;                           // Printing in progress.

    TMailer*        ApxMail;

//{{TBmpMenuSDIAppVIRTUAL_BEGIN}}
  public:
    virtual void InitMainWindow();
    virtual void InitInstance();
//{{TBmpMenuSDIAppVIRTUAL_END}}

//{{TBmpMenuSDIAppRSP_TBL_BEGIN}}
  protected:
    void EvNewView(TView& view);
    void EvCloseView(TView& view);
    void CeFileSend(TCommandEnabler& ce);
    void CmFileSend();
    void CmHelpAbout();
    void CmFilePrint();
    void CmFilePrintSetup();
    void CmFilePrintPreview();
    void CmPrintEnable(TCommandEnabler& tce);
    void EvWinIniChange(char far* section);
    void EvOwlDocument(TDocument& doc);
    int32 CmFileSelected(uint wp, int32 lp);
//{{TBmpMenuSDIAppRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(TBmpMenuSDIApp);
};    //{{TBmpMenuSDIApp}}


#endif  // bmpmenusdiapp_h sentry.
