/*
 * multisel.cpp
 *
 * Shows how to play with multiselection open dialog boxes using OWL framework
 * Copyright (C) 1997 Brice VIDAL
 *
 * Just more mumbo-jumbo...
 *
 * This program is free software; you can redistribute it and/or
 * modify it.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <owl/mdi.h>
#include <owl/opensave.h>

#include <dir.h>          // in order to get the directory
#include <cstring.h>      // useful when trying to play with strings !

class TYourMDIClient:public TMDIClient
{
//...
	private:
   	TOpenSaveDialog::TData FileData;
      void TYourMDIClient::OpenFile(const char* fileName = 0)
//...
};

void TYourMDIClient::SetupWindow()
{
  // Default SetUpWindow processing.
  //
  TMDIClient::SetupWindow();

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  FileData.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
  // Here wez set all the flags:
  //	OFN_FILEMUSTEXIST:     the file won't be created if it does not exists
  //  OFN_HIDEREADONLY:      hides the read only check box
  //  OFN_ALLOWMULTISELECT:  allows multiselection
  //  OFN_EXPLORER:          gives an explorer aspect = normal win95 dialogs with a combo box displaying the directory
  // For more flags see your help file in the TopenSaveDialog :: TData structure :: flags help file

  FileData.SetFilter("Your App Files (*.app)|*.app| RTF Files (*.rtf)|*.rtf| Text Files (*.txt)|*.txt| All Files (*.*)|*.*|");
  // Set the names that appears in the combo box then the exrension

  FileData.DefExt = "app";

  // Do what ever you want
// ...
}

void TYourMDIClient::OpenFile(const char* fileName)     // Opens the file
{
	if(!fileName)
   	// look in FileData.FileName for the file to open

   // Do what ever you want
// ...
}

void TYourMDIClient::CmFileOpen()       // responds to your Command Open file
{
  // Display standard Open dialog box to select a file name.
  //
  *(FileData.FileName) = 0;        // resets the FileName

  if (TFileOpenDialog(this, FileData).Execute() == IDOK)
  {
  		char curPath [MAXPATH];      // Get the current working directory
      getcwd(curPath, MAXPATH);    //

      int size = strlen(FileData.FileName);    //get the size, strlen counts the number
      			//of letter until it reaches a null terminator and the returns the number
               //of letters without counting the null terminator...

  		if(strlen(curPath) != size)   // If have they are different
      	OpenFile();                // there is only one file selected remember strlen properties...

      else                          // else there are several files selected
      {
       	char* path = FileData.FileName;

			bool exit = false; 			// need to know when we are going to get out from the loop

      	string dir(path);         // Initialize the string which represents the path
      	dir += "\\";              // add an antislash so that we only need to add the file name
      	while(!exit)
      	{
         /* As the FileName contains all the path but with null-terminated chars
            we only skeep the null-termination after the path*/
      		char* file = FileData.FileName + size + 1;

         	int fileSize = strlen(file); // take the size of the new char...

         	if(!fileSize)        // null-terminator => strlen() == 0
         		exit = true;      // No more files to open so exit

         	else
         	{
         		size += (fileSize + 1); // the new space to skip = the old space + the new one + 1 null-terminator

					string openName(file);       //create a string with the file name
         		openName = dir + openName;   // Copy the dir and the file name together
            	OpenFile(openName.c_str());  // Open the file
        		}
      	}// End of the while
      }// End of else = several files
  }// End of if = the user pressed the OK button
}

