/***********************************************************************************
 *		  																								     *
 *	File:			SB.cpp.			  					 												  *
 *																											  *
 * Contents: 	Implementation of TRange, TSpinEdit TSpinScrollBar and TSpinButton. *
 *																											  *
 * Remarks:    -												 										  *
 *																											  *
 * Originator: Michael Mogensen, Hilleroed, Denmark.										  *
 *																											  *
 *	E-mail:		mogensen at post6 dot tele dot dk.      												  *
 *																											  *
 * Compiler:	Borland C++ 5.01.																	  *
 *																											  *
 * Period:		98.07.10 - 98.07.13 																  *
 *																											  *
 ***********************************************************************************/

/***********************************************************************************
 *	Headers.															 									  *
 ***********************************************************************************/
#pragma hdrstop
#include "SB.h"

/***********************************************************************************
 *	TRange.														 										  *
 ***********************************************************************************/

// 1. Object. (alphabetical).
TRange::TRange(const double dBottomEx,
       			const double dCurrentEx,
       			const double dTopEx,
       			const double dStepEx,
               const int iPrecisionEx) :
		 			dBottom(dBottomEx),
     	 			dCurrent(dCurrentEx),
          		dTop(dTopEx),
  		 	 		dStep(dStepEx),
               iPrecision(iPrecisionEx),
               bTouchZero(false)
   // Create.
	{
   	pszCurrent = new char[80];
      if(pszCurrent)
      	strcpy(pszCurrent, "");
      Normalize();
   }

TRange::~TRange()
// Delete.
{
	if(pszCurrent)
   	delete [] pszCurrent;
}

// 4. Slaves. (alphabetical).
const void TRange::Normalize()
{
	if(dBottom > dTop)
		SwapObjects(dBottom, dTop);
   SupressObjectInRange(dCurrent, dBottom, dTop);
   dStep = Abs(dStep);
   dStep = max(I_DStepMin, dStep);
   if(dStep > (dTop - dBottom) / 2.0)
   	dStep = I_DStep;
}

/***********************************************************************************
 *	TSpinEdit.														 									  *
 ***********************************************************************************/

// 1. Object. (alphabetical).
TSpinEdit::TSpinEdit(TWindow *pParent,
      				   const int iId,
		   				const double dBottom,
      		    		const double dCurrent,
          				const double dTop,
		          		const double dStep,
      		    		const int iPrecision,
               		const TCharSet Validator) :
                     TEdit(pParent, iId - 1)
   // Create.
	{
		pRange = new TRange(dBottom, dCurrent, dTop, dStep, iPrecision);
   	if(!Validator.IsEmpty())
   		SetValidator(new TFilterValidator(Validator));
   }

TSpinEdit::~TSpinEdit()
// Delete.
{
	if(pRange)
   	delete pRange;
}

// 2. Event's.
void TSpinEdit::SetupWindow()
{
	TEdit::SetupWindow();
   SetCurrent(pRange->dCurrent);
}

/***********************************************************************************
 *	TSpinScrollBar.												 									  *
 ***********************************************************************************/

DEFINE_RESPONSE_TABLE1(TSpinScrollBar, TScrollBar)
	EV_WM_HSCROLL,
	EV_WM_VSCROLL,
END_RESPONSE_TABLE;

// 1. Object. (alphabetical).
TSpinScrollBar::TSpinScrollBar(TWindow *pParent,
										 const int iId,
				   					 const double dBottom,
          							 const double dCurrent,
				         			 const double dTop,
          							 const double dStep,
				         			 const int iPrecision,
                               const TCharSet Validator) :
                     			 TScrollBar(pParent, iId + 1)
   // Create.
	{ pSpinEdit = new TSpinEdit(pParent, iId, dBottom, dCurrent, dTop, dStep, iPrecision, Validator); }

TSpinScrollBar::~TSpinScrollBar()
// Delete.
{
	if(pSpinEdit)
   	delete pSpinEdit;
}

// 2. Event's.
void TSpinScrollBar::SetupWindow()
// Setup.
{
	TScrollBar::SetupWindow();
}

void TSpinScrollBar::EvHScroll(UINT uScrollCode, UINT uThumbPosition, HWND hWindow)
// Respond to left/right events (for special spinner look).
{
	TScrollBar::EvHScroll(uScrollCode, uThumbPosition, hWindow);
	switch(uScrollCode)
	{
		case SB_LINEUP: (*pSpinEdit)--; return;
		case SB_LINEDOWN: (*pSpinEdit)++; return;
	}
}

void TSpinScrollBar::EvVScroll(UINT uScrollCode, UINT uThumbPosition, HWND hWindow)
// Respond to up/down events (for normal spinner look).
{
	TScrollBar::EvVScroll(uScrollCode, uThumbPosition, hWindow);
	switch(uScrollCode)
	{
		case SB_LINEUP: (*pSpinEdit)++; return;
		case SB_LINEDOWN: (*pSpinEdit)--; return;
	}
}

/***********************************************************************************
 *	TSpinButton.													 									  *
 ***********************************************************************************/

// 1. Object. (alphabetical).
TSpinButton::TSpinButton(TWindow *pParent,
								 const int iId,
                  		 const double dBottom,
								 const double dCurrent,
                  		 const double dTop,
                   		 const double dStep,
                  		 const int iPrecision,
                  		 const TCharSet Validator)
	// Create.
	{ pSpinScrollBar = new TSpinScrollBar(pParent, iId, dBottom, dCurrent, dTop, dStep, iPrecision, Validator); }

TSpinButton::~TSpinButton()
// Delete.
{
	if(pSpinScrollBar)
   	delete pSpinScrollBar;
}

/***********************************************************************************
 *																		 									  *
 ***********************************************************************************/

// 0. Data. (alphabetical).
// 1. Object. (alphabetical).
// 2. Event's. (alphabetical).
// 3.0. Menu choice. (menu order).
// 3.1. Menu choice, enablers. (menu order).
// 4. Slaves. (alphabetical).
// 5. Other. (alphabetical).

