/***********************************************************************************
 *		  																								     *
 *	File:			SB.h.			  				 														  *
 *																											  *
 * Contents: 	Interface of TRange, TSpinEdit TSpinScrollBar and TSpinButton. 	  *
 *																											  *
 * Purpose:  	Be able to spin with doubles.													  *
 *																											  *
 * Remarks:    -												 										  *
 *																											  *
 * Originator: Michael Mogensen, Hilleroed, Denmark.										  *
 *																											  *
 *	E-mail:		mogensen at post6 dot tele dot dk.      												  *
 *																											  *
 * Compiler:	Borland C++ 5.01.																	  *
 *																											  *
 * Period:		98.07.10 - 98.07.13 																  *
 *																											  *
 ***********************************************************************************/

/***********************************************************************************
 *	Miscellaneous.																					 	  *
 ***********************************************************************************/
#define STRICT

#ifndef SB_h
#define SB_h

/***********************************************************************************
 *	Headers, OWL.													 									  *
 ***********************************************************************************/
#include <owl\bitset.h>					// TCharSet.
#include <owl\edit.h>     				// TEdit.
#include <owl\scrollba.h>     		// TScrollBar.
#include <owl\validate.h>     		// TFilterValidator.

/***********************************************************************************
 *	Headers, C.													 										  *
 ***********************************************************************************/
#include <stdlib.h>						// max(), min() etc.

/***********************************************************************************
 *	Private.																 								  *
 ***********************************************************************************/
#include "generic.h"                // Templates.

/***********************************************************************************
 *	Defaults.														 									  *
 ***********************************************************************************/
#define I_DBottom 0.0
#define I_DCurrent 0.0
#define I_DTop 100.0
#define I_DStep 1.0
#define I_DStepMin 1.0e-10
#define I_IPrecision 12
#define I_SValidator "0-9.\-"

/***********************************************************************************
 *	TRange.															 								 	  *
 ***********************************************************************************/
struct TRange
{
	// 0. Data. (alphabetical).
	bool bTouchZero;
   char *pszCurrent;
	double dBottom,
     		 dCurrent,
          dTop,
  		 	 dStep;
   const int iPrecision;
	// 1. Object. (alphabetical).
   TRange(const double dBottomEx = I_DBottom,
          const double dCurrentEx = I_DCurrent,
          const double dTopEx = I_DTop,
          const double dStepEx = I_DStep,
          const int iPrecisionEx = I_IPrecision);
   ~TRange();
	// 4. Slaves. (alphabetical).
	const void Normalize();
};

/***********************************************************************************
 *	TSpinEdit.															 								  *
 ***********************************************************************************/
class TSpinEdit : public TEdit
{
	protected:
		// 0. Data. (alphabetical).
       char *pszHelper;
		// 2. Event's. (alphabetical).
		virtual void SetupWindow();
	public:
		// 0. Data. (alphabetical).
      TRange *pRange;
		// 1. Object. (alphabetical).
		TSpinEdit(TWindow *pParent,
      			 const int iId,
   				 const double dBottomEx = I_DBottom,
          		 const double dCurrentEx = I_DCurrent,
          		 const double dTopEx = I_DTop,
          		 const double dStepEx = I_DStep,
          		 const int iPrecisionEx = I_IPrecision,
                const TCharSet ValidatorEx = I_SValidator);
      ~TSpinEdit();
		// 4. Slaves. (alphabetical).
      inline const TSpinEdit& operator++(int)
      {
			if(IsModified())
         {
			  	TEdit::GetText(pRange->pszCurrent, 80);
            SetCurrent(strtod(pRange->pszCurrent, &pszHelper) + pRange->dStep);
         }
         else
       		SetCurrent(pRange->dCurrent + pRange->dStep);
         return *this;
      }
      inline const TSpinEdit& operator--(int)
      {
			if(IsModified())
         {
			  	TEdit::GetText(pRange->pszCurrent, 80);
            SetCurrent(strtod(pRange->pszCurrent, &pszHelper) - pRange->dStep);
         }
         else
	       	SetCurrent(pRange->dCurrent - pRange->dStep);
         return *this;
      }
      inline const HWND GetHandle()
      { return TWindow::GetHandle(); }
		inline const void SetCurrent(const double dCurrentEx)
      {
      	pRange->dCurrent = dCurrentEx;
     		SupressObjectInRange(pRange->dCurrent, pRange->dBottom, pRange->dTop);
         if(pRange->bTouchZero && Abs(pRange->dCurrent) < pRange->dStep || Abs(pRange->dCurrent) < I_DStepMin)
         	pRange->dCurrent = 0.0;
         gcvt(pRange->dCurrent, pRange->iPrecision, pRange->pszCurrent);
       	SetText(pRange->pszCurrent);
      }
		// 4. Slaves. (alphabetical).
      inline const void SetHandle(HWND hWnd)
      { TWindow::SetHandle(hWnd); }
};

/***********************************************************************************
 *	TSpinScrollBar.														 							  *
 ***********************************************************************************/
class TSpinScrollBar : public TScrollBar
{
	protected:
		// 2. Event's. (alphabetical).
      virtual void EvHScroll(UINT, UINT, HWND);
      virtual void EvVScroll(UINT, UINT, HWND);
		virtual void SetupWindow();
   public:
		// 0. Data. (alphabetical).
      TSpinEdit *pSpinEdit;
		// 1. Object. (alphabetical).
   	TSpinScrollBar(TWindow *pParent,
							const int iId,
				   		const double dBottomEx = I_DBottom,
          				const double dCurrentEx = I_DCurrent,
				         const double dTopEx = I_DTop,
          				const double dStepEx = I_DStep,
				         const int iPrecisionEx = I_IPrecision,
                     const TCharSet ValidatorEx = I_SValidator);
      ~TSpinScrollBar();
      inline const void SetHandle(HWND hWnd)
      { TWindow::SetHandle(hWnd); }
	// 5. Other. (alphabetical).
   DECLARE_RESPONSE_TABLE(TSpinScrollBar);
};

/***********************************************************************************
 *	TSpinEdit.															 								  *
 ***********************************************************************************/
class TSpinButton
{
	protected:
		// 0. Data. (alphabetical).
      TSpinScrollBar *pSpinScrollBar;
   public:
		// 1. Object. (alphabetical).
   	TSpinButton(TWindow *pParent,
					   const int iId,
				      const double dBottomEx = I_DBottom,
          		   const double dCurrentEx = I_DCurrent,
				      const double dTopEx = I_DTop,
          		   const double dStepEx = I_DStep,
				      const int iPrecisionEx = I_IPrecision,
                  const TCharSet ValidatorEx = I_SValidator);
		~TSpinButton();
		// 4. Slaves. (alphabetical).
		inline const void Enable(const bool bState)
      {
      	pSpinScrollBar->EnableWindow(bState);
      	pSpinScrollBar->pSpinEdit->EnableWindow(bState);
      }
		inline const double& GetBottom()
      { return pSpinScrollBar->pSpinEdit->pRange->dBottom; }
		inline const double& GetCurrent()
      { return pSpinScrollBar->pSpinEdit->pRange->dCurrent; }
      inline const HWND GetHandleEdit()
      { return pSpinScrollBar->pSpinEdit->GetHandle(); }
      inline const HWND GetHandleScroller()
      { return pSpinScrollBar->GetHandle(); }
		inline const double GetRange()
      { return(pSpinScrollBar->pSpinEdit->pRange->dTop - pSpinScrollBar->pSpinEdit->pRange->dBottom); }
		inline const void GetRange(double &dBottomEx, double &dTopEx)
      {
      	dBottomEx = pSpinScrollBar->pSpinEdit->pRange->dBottom;
         dTopEx = pSpinScrollBar->pSpinEdit->pRange->dTop;
      }
		inline const double& GetTop()
      { return pSpinScrollBar->pSpinEdit->pRange->dTop; }
		inline const double& GetStep()
      { return pSpinScrollBar->pSpinEdit->pRange->dStep; }
		inline const char* GetText()
      { return pSpinScrollBar->pSpinEdit->pRange->pszCurrent; }
		inline const void SetBottom(const double &dBottomEx)
      {
      	pSpinScrollBar->pSpinEdit->pRange->dBottom = dBottomEx;
      	pSpinScrollBar->pSpinEdit->pRange->Normalize();
      }
		inline const void SetCurrent(const double &dCurrentEx)
      { pSpinScrollBar->pSpinEdit->SetCurrent(dCurrentEx); }
      inline const void SetHandleEdit(HWND hWnd)
      { pSpinScrollBar->pSpinEdit->SetHandle(hWnd); }
      inline const void SetHandleScroller(HWND hWnd)
      { pSpinScrollBar->SetHandle(hWnd); }
		inline const void SetRange(double &dBottomEx, double &dTopEx)
      {
      	pSpinScrollBar->pSpinEdit->pRange->dBottom = dBottomEx;
         pSpinScrollBar->pSpinEdit->pRange->dTop = dTopEx;
      	pSpinScrollBar->pSpinEdit->pRange->Normalize();
      }
		inline const void SetTop(const double &dTopEx)
      {
      	pSpinScrollBar->pSpinEdit->pRange->dTop = dTopEx;
      	pSpinScrollBar->pSpinEdit->pRange->Normalize();
      }
		inline const void SetStep(double &dStepEx)
      {
      	pSpinScrollBar->pSpinEdit->pRange->dStep = dStepEx;
      	pSpinScrollBar->pSpinEdit->pRange->Normalize();
      }
	   inline const void TouchZero(const bool bTouchZeroEx)
   	{ pSpinScrollBar->pSpinEdit->pRange->bTouchZero = bTouchZeroEx; }
      inline const void Update()
      { pSpinScrollBar->pSpinEdit->SetText(GetText()); }
};

/***********************************************************************************
 *			 															 									  *
 ***********************************************************************************/


// 0. Data. (alphabetical).
// 1. Object. (alphabetical).
// 2. Event's. (alphabetical).
// 3.0. Menu choice. (menu order).
// 3.1. Menu choice, enablers. (menu order).
// 4. Slaves. (alphabetical).
// 5. Other. (alphabetical).

#endif

