#include <owl/pch.h>


// Our client window class is called TTutorialClientWindow
// and is inherited from TWindow. At this stage it does
// very little of interest.
//
class TTutorialClientWindow : public TWindow
{
  public:
    TTutorialClientWindow(TWindow* parent, TModule* module = 0);
};


// Client Window Constructor
// ~~~~~~~~~~~~~~~~~~~~~~~~~
// This constructor currently does little more than
// call the appropriate base class (TWindow) constructor.
//
TTutorialClientWindow::TTutorialClientWindow(TWindow* parent, TModule* module)
: TWindow(parent, 0 /* window does not need a title */, module)
{
}


// Our application class is called TTutorialApp
// and is inherited from TApplication.
//
class TTutorialApp : public TApplication
{
  public:
    TTutorialApp();
    void InitMainWindow();
};


// Application Constructor
// ~~~~~~~~~~~~~~~~~~~~~~~
// All we are doing here is telling OWL what we
// want the name of our application to be.
//
TTutorialApp::TTutorialApp()
: TApplication("Tutorial Application")
{
}


// Create the Main Window
// ~~~~~~~~~~~~~~~~~~~~~~
// InitMainWindow is a virtual function in TApplication.
// We override it to create our own main window and its
// client window.
//
void TTutorialApp::InitMainWindow()
{
  // Create the client window.
  //
  // The first argument to the constructor of a window
  // class is usually a pointer to a parent window.
  // However, when creating a client window we do not
  // specify a parent since the frame window will do
  // that for us automatically.
  //
  TTutorialClientWindow* client = new TTutorialClientWindow(0);

  // Create the frame window.
  //
  // As above, we do not supply a parent window, but this
  // time it is because a main window has no parent.
  //
  // The second argument is the title that we want for the
  // main window (it is displayed in the caption bar). In
  // this case we will use the name that we gave to the
  // application.
  //
  // The third argument is the client window.
  //
  TFrameWindow* frame = new TFrameWindow(0, GetName(), client);

  // Finally, we tell OWL that we want frame to be our
  // main window.
  //
  SetMainWindow(frame);
}


// OwlMain
// ~~~~~~~
// OwlMain is where it all starts. You use OwlMain to
// create your application object and run it.
//
int OwlMain(int /*argc*/, char* /*argv*/[])
{
  TTutorialApp app;
  return app.Run();
}
