#include <owl/pch.h>
#include <stdio.h>

#include "step2.rh"


// -----------------------------------------------------------
// TTutorialClientWindow class
//

class TTutorialClientWindow : public TWindow
{
  public:
    TTutorialClientWindow(TWindow* parent, TModule* module = 0);

  public:
    enum TShapeType {
      Line      = CM_SHAPELINE,
      Rectangle = CM_SHAPERECTANGLE,
      Ellipse   = CM_SHAPEELLIPSE
    };
    static TShapeType ShapeType;

  protected:
    void EvLButtonDown(uint modKeys, TPoint& point);
    void EvLButtonUp(uint modKeys, TPoint& point);
    void EvMouseMove(uint modKeys, TPoint& point);
    void CmShapeClear();
    DECLARE_RESPONSE_TABLE(TTutorialClientWindow);
};


TTutorialClientWindow::TShapeType TTutorialClientWindow::ShapeType = Line;


// Define our window response table.
//
// Note how there is a '1' at the end of macro name. This
// indicates that the class TTutorialClientWindow has one
// base class that also has a response table.
//
// The first parameter to the DEFINE_RESPONSE_TABLEx macro
// is the name of the class that owns the response table.
//
// Subsequent arguments are immediate base classes that
// also have a response table (and should therefore have
// events forwarded to them).
//
DEFINE_RESPONSE_TABLE1(TTutorialClientWindow, TWindow)

  // Add a handler for the left mouse-button down event.
  // There must be a function called EvLButtonDown to
  // handle the message.
  //
  EV_WM_LBUTTONDOWN,

  // Add a handler for the left mouse-button up event.
  // There must be a function called EvLButtonUp to
  // handle the message.
  //
  EV_WM_LBUTTONUP,

  // Add a handler for the mouse move event. There
  // must be a function called EvLButtonDown to handle
  // the message.
  //
  EV_WM_MOUSEMOVE,

  // Associate the menu item Shape|Clear with the
  // function CmShapeClear. The first parameter is
  // the id of the menu item (see the menu resource).
  // The second parameter is the corresponding member
  // function in the TTutorialClientWindow class.
  //
  EV_COMMAND(CM_SHAPECLEAR, CmShapeClear),

END_RESPONSE_TABLE;


TTutorialClientWindow::TTutorialClientWindow(TWindow* parent, TModule* module)
: TWindow(parent, 0, module)
{
}


void TTutorialClientWindow::EvLButtonDown(uint /*modKeys*/, TPoint& /*point*/)
{
}


void TTutorialClientWindow::EvLButtonUp(uint /*modKeys*/, TPoint& /*point*/)
{
}


void TTutorialClientWindow::EvMouseMove(uint /*modKeys*/, TPoint& /*point*/)
{
}


void TTutorialClientWindow::CmShapeClear()
{
  MessageBox("Clear Shapes");
}


// -----------------------------------------------------------
// TTutorialApp class
//

class TTutorialApp : public TApplication
{
  public:
    TTutorialApp();
    void InitMainWindow();

  protected:
    void CmHelpAbout();
    void CmShapeSelect(uint id);
    void CeShapeSelect(TCommandEnabler& tce);
    DECLARE_RESPONSE_TABLE(TTutorialApp);
};


// Define our application response table.
//
DEFINE_RESPONSE_TABLE1(TTutorialApp, TApplication)

  // Associate the Help|About menu command
  // with the function CmHelpAbout.
  //
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),

  // Add handlers so that one and only one of
  // Shape|Line, Shape|Rectangle or Shape|Ellipse
  // will be selected at any given time.
  //
  EV_COMMAND_AND_ID(CM_SHAPELINE, CmShapeSelect),
  EV_COMMAND_AND_ID(CM_SHAPERECTANGLE, CmShapeSelect),
  EV_COMMAND_AND_ID(CM_SHAPEELLIPSE, CmShapeSelect),
  EV_COMMAND_ENABLE(CM_SHAPELINE, CeShapeSelect),
  EV_COMMAND_ENABLE(CM_SHAPERECTANGLE, CeShapeSelect),
  EV_COMMAND_ENABLE(CM_SHAPEELLIPSE, CeShapeSelect),

END_RESPONSE_TABLE;


TTutorialApp::TTutorialApp()
: TApplication("Tutorial Application")
{
}


void TTutorialApp::InitMainWindow()
{
  TTutorialClientWindow* client = new TTutorialClientWindow(0);
  TFrameWindow* frame = new TFrameWindow(0, GetName(), client);

  // Set the menu for the main window.
  //
  frame->AssignMenu(IDM_MAINMENU);

  // Associate a menu accelerator (shortcut keys)
  // table with the main window.
  //
  frame->Attr.AccelTable = IDM_MAINMENU;

  SetMainWindow(frame);
}


void TTutorialApp::CmHelpAbout()
{
  MessageBox(0, "About", "Help", MB_OK);
}


void TTutorialApp::CmShapeSelect(uint id)
{
  TTutorialClientWindow::ShapeType = (TTutorialClientWindow::TShapeType) id;
}


void TTutorialApp::CeShapeSelect(TCommandEnabler& tce)
{
  // Set the check of the menu item depending on whether
  // or not it was the item most recently selected.
  //
  tce.SetCheck(TTutorialClientWindow::ShapeType == tce.GetId() ?
               TCommandEnabler::Checked :
               TCommandEnabler::Unchecked);
}


int OwlMain(int /*argc*/, char* /*argv*/[])
{
  TTutorialApp app;
  return app.Run();
}
