/******************************************************************************/
// modul: fileutil.h   version 1.0b (english)                                 //
//                                                                            //
//  utility functions to read values/strings from a stream. you can realize   //
//  format-sensitive reading without the "always read a whoole line" concept. //
//                                                                            //
//  written by lore@newty.de   -   home: www.newty.de                         //
//  copyright 1998-2000 by lore                                               //
//                                                                            //
//  FREE for all kinds of non-commercial use including modifying and          //
//  redistribution as long, as this header is not removed!                    //
//  disclaimer: nothing is guaranteed. you use it on your own risk.           //
//                                                                            //
//  please report bugs immediately to bugs@newty.de !!                        //
//                                                                            //
/******************************************************************************/


#include <fstream.h>					// due to: 	ifstream ...


/***********************************************************************************************************/
// exception codes and functions to retrieve an error-describing text or the occured illegal character

#define errUnexpectedEndl 			42				// encountered unexpected end of line (endl)
#define errUnexpectedEOF 			43				//      "           "     end of file (EOF)
#define errNoEndlFound     		44    		// an endl was expected but not found
#define errCharacterIllegal   	45				// encountered illegal character
#define errNoWhitespace				46				// no whitespace after value


const char* getErrCode(const int errNo);		// get error-text
char& 		illChar();


/***********************************************************************************************************/
// functions skipws()/skipwsEx() and wrappers

// functions to remove whitespaces out of an ifstream returning the number of encountered endls
// the extended version also removes all comments
int skipws	(ifstream& file);
int skipwsEx(ifstream& file);


// wrapper for the two functions above which are throwing an exception depending on the number of
// encountered endls
int 	skipwsEndl		(ifstream& file);		// use skipws() throwing exception if no endl occurs
void 	skipwsNoEndl	(ifstream& file);		//  "     "        "         "      " an  "     "

int 	skipwsExEndl	(ifstream& file);		// like above but use skipwsEx()
void 	skipwsExNoEndl	(ifstream& file);		//  "     "    "   "      "



/***********************************************************************************************************/
// "read"-functions

	// reading a string from an ifstream. the characters '=' and ':' and all whitespaces terminate the string.
	char readString(ifstream& file, char* string, const int len);


	/*********************************************************************************************/
	// functions reads an object of type T from an ifstream throwing an exception if the
   // reading causes a failstate of the stream. note: the '>>' operator skips whitespaces like
   // endls. thus if you track the actual line-number in the file dont forget to call
   // skipws()/skipwsEx() before.

		void readFailed(ifstream& file);		// utility function - private :-))

	template <class T> T read(ifstream& file, const T)
	{
	 	file.clear();								// reset stream state

	 	T value;
   	file >> value;								// try to read from stream

	   if (file.fail()!=0)						// examine in case of a failstate
   		readFailed(file);

	   if(!isspace(file.peek()))				// the next character after the value should be a ws ...
		{
	   	illChar() = (char) file.peek();  // set as "illegal character"
   		throw errNoWhitespace;				// ... otherwise: throw exception
	   }

   	return value;
	}


/***********************************************************************************************************/
// counts the number of endls in an ifstream ignoring empty lines or comments.
int countLinesEx(ifstream& file);

